@@ -33,7 +33,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
       Key.WRITE_HISTORY, Key.DIRREQ_READ_HISTORY, Key.DIRREQ_WRITE_HISTORY,
       Key.GEOIP_DB_DIGEST, Key.GEOIP6_DB_DIGEST, Key.ROUTER_SIG_ED25519,
       Key.ROUTER_SIGNATURE, Key.ROUTER_DIGEST_SHA256, Key.ROUTER_DIGEST,
-      Key.PADDING_COUNTS);
+      Key.PADDING_COUNTS, Key.OVERLOAD_RATELIMITS, Key.OVERLOAD_FD_EXHAUSTED);
 
   protected ExtraInfoDescriptorImpl(byte[] descriptorBytes,
       int[] offsetAndLimit, File descriptorFile)
@@ -97,6 +97,12 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         case WRITE_HISTORY:
           this.parseWriteHistoryLine(line, partsNoOpt);
           break;
+        case OVERLOAD_RATELIMITS:
+          this.parseOverloadRatelimits(line, partsNoOpt);
+          break;
+        case OVERLOAD_FD_EXHAUSTED:
+          this.parseOverloadFdExhausted(line, partsNoOpt);
+          break;
         case IPV6_READ_HISTORY:
           this.parseIpv6ReadHistoryLine(line, partsNoOpt);
           break;
@@ -225,6 +231,15 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         case HIDSERV_DIR_ONIONS_SEEN:
           this.parseHidservDirOnionsSeenLine(line, partsNoOpt);
           break;
+        case HIDSERV_V3_STATS_END:
+          this.parseHidservV3StatsEndLine(line, partsNoOpt);
+          break;
+        case HIDSERV_REND_V3_RELAYED_CELLS:
+          this.parseHidservRendV3RelayedCellsLine(line, partsNoOpt);
+          break;
+        case HIDSERV_DIR_V3_ONIONS_SEEN:
+          this.parseHidservDirV3OnionsSeenLine(line, partsNoOpt);
+          break;
         case PADDING_COUNTS:
           this.parsePaddingCountsLine(line, partsNoOpt);
           break;
@@ -323,6 +338,41 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         partsNoOpt);
   }
 
+  private void parseOverloadRatelimits(String line,
+      String[] partsNoOpt) throws DescriptorParseException {
+    this.overloadRatelimitsVersion = Integer.parseInt(partsNoOpt[1]);
+    if (this.overloadRatelimitsVersion != 1) {
+      throw new DescriptorParseException("Unknown version number for line '"
+              + line + "' in extra-info descriptor.");
+    }
+    if (partsNoOpt.length < 8) {
+      throw new DescriptorParseException("Missing fields for line '"
+          + line + "' in extra-info descriptor.");
+    }
+    this.overloadRatelimitsTimestamp = ParseHelper.parseTimestampAtIndex(line,
+        partsNoOpt, 2, 3);
+    this.overloadRatelimitsRateLimit = Long.parseLong(partsNoOpt[4]);
+    this.overloadRatelimitsBurstLimit = Long.parseLong(partsNoOpt[5]);
+    this.overloadRatelimitsReadCount = Integer.parseInt(partsNoOpt[6]);
+    this.overloadRatelimitsWriteCount = Integer.parseInt(partsNoOpt[7]);
+  }
+
+  private void parseOverloadFdExhausted(String line,
+                                       String[] partsNoOpt)
+      throws DescriptorParseException {
+    this.overloadFdExhaustedVersion = Integer.parseInt(partsNoOpt[1]);
+    if (this.overloadFdExhaustedVersion != 1) {
+      throw new DescriptorParseException("Unknown version number for line '"
+              + line + "' in extra-info descriptor.");
+    }
+    if (partsNoOpt.length < 4) {
+      throw new DescriptorParseException("Missing fields for line '"
+          + line + "' in extra-info descriptor.");
+    }
+    this.overloadFdExhaustedTimestamp = ParseHelper.parseTimestampAtIndex(line,
+            partsNoOpt, 2, 3);
+  }
+
   private void parseIpv6ReadHistoryLine(String line,
       String[] partsNoOpt) throws DescriptorParseException {
     this.ipv6ReadHistory = new BandwidthHistoryImpl(line,
@@ -764,6 +814,46 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         partsNoOpt, 2);
   }
 
+  private void parseHidservV3StatsEndLine(String line,
+      String[] partsNoOpt) throws DescriptorParseException {
+    long[] parsedStatsEndData = this.parseStatsEndLine(line, partsNoOpt,
+        5);
+    this.hidservV3StatsEndMillis = parsedStatsEndData[0];
+    this.hidservV3StatsIntervalLength = parsedStatsEndData[1];
+  }
+
+  private void parseHidservRendV3RelayedCellsLine(String line,
+      String[] partsNoOpt)
+      throws DescriptorParseException {
+    if (partsNoOpt.length < 2) {
+      throw new DescriptorParseException("Illegal line '" + line + "'.");
+    }
+    try {
+      this.hidservRendV3RelayedCells = Double.parseDouble(partsNoOpt[1]);
+    } catch (NumberFormatException e) {
+      throw new DescriptorParseException("Illegal line '" + line + "'.");
+    }
+    this.hidservRendV3RelayedCellsParameters =
+        ParseHelper.parseSpaceSeparatedStringKeyDoubleValueMap(line,
+            partsNoOpt, 2);
+  }
+
+  private void parseHidservDirV3OnionsSeenLine(String line,
+      String[] partsNoOpt)
+      throws DescriptorParseException {
+    if (partsNoOpt.length < 2) {
+      throw new DescriptorParseException("Illegal line '" + line + "'.");
+    }
+    try {
+      this.hidservDirV3OnionsSeen = Double.parseDouble(partsNoOpt[1]);
+    } catch (NumberFormatException e) {
+      throw new DescriptorParseException("Illegal line '" + line + "'.");
+    }
+    this.hidservDirV3OnionsSeenParameters =
+        ParseHelper.parseSpaceSeparatedStringKeyDoubleValueMap(line,
+            partsNoOpt, 2);
+  }
+
   private void parsePaddingCountsLine(String line,
       String[] partsNoOpt) throws DescriptorParseException {
     long[] parsedStatsEndData = this.parseStatsEndLine(line, partsNoOpt,
@@ -904,6 +994,62 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
     return this.geoip6DbDigest;
   }
 
+  private int overloadRatelimitsVersion = 0;
+
+  @Override
+  public long getOverloadRatelimitsVersion() {
+    return this.overloadRatelimitsVersion;
+  }
+
+  private long overloadRatelimitsTimestamp = -1L;
+
+  @Override
+  public long getOverloadRatelimitsTimestamp() {
+    return this.overloadRatelimitsTimestamp;
+  }
+
+  private long overloadRatelimitsRateLimit = -1L;
+
+  @Override
+  public long getOverloadRatelimitsRateLimit() {
+    return this.overloadRatelimitsRateLimit;
+  }
+
+  private long overloadRatelimitsBurstLimit = -1L;
+
+  @Override
+  public long getOverloadRatelimitsBurstLimit() {
+    return this.overloadRatelimitsBurstLimit;
+  }
+
+  private int overloadRatelimitsReadCount = -1;
+
+  @Override
+  public int getOverloadRatelimitsReadCount() {
+    return this.overloadRatelimitsReadCount;
+  }
+
+  private int overloadRatelimitsWriteCount = -1;
+
+  @Override
+  public int getOverloadRatelimitsWriteCount() {
+    return this.overloadRatelimitsWriteCount;
+  }
+
+  private int overloadFdExhaustedVersion = 0;
+
+  @Override
+  public int getOverloadFdExhaustedVersion() {
+    return this.overloadFdExhaustedVersion;
+  }
+
+  private long overloadFdExhaustedTimestamp = -1L;
+
+  @Override
+  public long getOverloadFdExhaustedTimestamp() {
+    return this.overloadFdExhaustedTimestamp;
+  }
+
   private long dirreqStatsEndMillis = -1L;
 
   @Override
@@ -1319,6 +1465,50 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         : new HashMap<>(this.hidservDirOnionsSeenParameters);
   }
 
+  private long hidservV3StatsEndMillis = -1L;
+
+  @Override
+  public long getHidservV3StatsEndMillis() {
+    return this.hidservV3StatsEndMillis;
+  }
+
+  private long hidservV3StatsIntervalLength = -1L;
+
+  @Override
+  public long getHidservV3StatsIntervalLength() {
+    return this.hidservV3StatsIntervalLength;
+  }
+
+  private Double hidservRendV3RelayedCells;
+
+  @Override
+  public Double getHidservRendV3RelayedCells() {
+    return this.hidservRendV3RelayedCells;
+  }
+
+  private Map<String, Double> hidservRendV3RelayedCellsParameters;
+
+  @Override
+  public Map<String, Double> getHidservRendV3RelayedCellsParameters() {
+    return this.hidservRendV3RelayedCellsParameters == null ? null
+        : new HashMap<>(this.hidservRendV3RelayedCellsParameters);
+  }
+
+  private Double hidservDirV3OnionsSeen;
+
+  @Override
+  public Double getHidservDirV3OnionsSeen() {
+    return this.hidservDirV3OnionsSeen;
+  }
+
+  private Map<String, Double> hidservDirV3OnionsSeenParameters;
+
+  @Override
+  public Map<String, Double> getHidservDirV3OnionsSeenParameters() {
+    return this.hidservDirV3OnionsSeenParameters == null ? null
+        : new HashMap<>(this.hidservDirV3OnionsSeenParameters);
+  }
+
   private long paddingCountsStatsEndMillis = -1L;
 
   @Override
@@ -1369,4 +1559,3 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
     return this.routerSignatureEd25519;
   }
 }
-
