@@ -7,7 +7,7 @@ public enum Key {
 
   EMPTY("the-empty-key"),
   INVALID("the-invalid-key"),
-  
+
   /* crypto keys */
   CRYPTO_BEGIN("-----BEGIN"),
   CRYPTO_END("-----END"),
@@ -17,7 +17,16 @@ public enum Key {
   ACCEPT("accept"),
   ALLOW_SINGLE_HOP_EXITS("allow-single-hop-exits"),
   BANDWIDTH("bandwidth"),
+  BANDWIDTH_FILE_DIGEST("bandwidth-file-digest"),
+  BANDWIDTH_FILE_HEADERS("bandwidth-file-headers"),
   BANDWIDTH_WEIGHTS("bandwidth-weights"),
+  BRIDGEDB_METRICS_END("bridgedb-metrics-end"),
+  BRIDGEDB_METRICS_VERSION("bridgedb-metrics-version"),
+  BRIDGEDB_METRIC_COUNT("bridgedb-metric-count"),
+  BRIDGESTRAP_STATS_END("bridgestrap-stats-end"),
+  BRIDGESTRAP_CACHED_REQUESTS("bridgestrap-cached-requests"),
+  BRIDGESTRAP_TEST("bridgestrap-test"),
+  BRIDGE_DISTRIBUTION_REQUEST("bridge-distribution-request"),
   BRIDGE_IPS("bridge-ips"),
   BRIDGE_IP_TRANSPORTS("bridge-ip-transports"),
   BRIDGE_IP_VERSIONS("bridge-ip-versions"),
@@ -30,7 +39,9 @@ public enum Key {
   CELL_STATS_END("cell-stats-end"),
   CELL_TIME_IN_QUEUE("cell-time-in-queue"),
   CLIENT_DENIED_COUNT("client-denied-count"),
+  CLIENT_RESTRICTED_DENIED_COUNT("client-restricted-denied-count"),
   CLIENT_SNOWFLAKE_MATCH_COUNT("client-snowflake-match-count"),
+  CLIENT_UNRESTRICTED_DENIED_COUNT("client-unrestricted-denied-count"),
   CLIENT_VERSIONS("client-versions"),
   CONN_BI_DIRECT("conn-bi-direct"),
   CONSENSUS_METHOD("consensus-method"),
@@ -84,11 +95,17 @@ public enum Key {
   HIBERNATING("hibernating"),
   HIDDEN_SERVICE_DIR("hidden-service-dir"),
   HIDSERV_DIR_ONIONS_SEEN("hidserv-dir-onions-seen"),
+  HIDSERV_DIR_V3_ONIONS_SEEN("hidserv-dir-v3-onions-seen"),
   HIDSERV_REND_RELAYED_CELLS("hidserv-rend-relayed-cells"),
+  HIDSERV_REND_V3_RELAYED_CELLS("hidserv-rend-v3-relayed-cells"),
   HIDSERV_STATS_END("hidserv-stats-end"),
+  HIDSERV_V3_STATS_END("hidserv-v3-stats-end"),
   ID("id"),
   IDENTITY_ED25519("identity-ed25519"),
+  IPV6_CONN_BI_DIRECT("ipv6-conn-bi-direct"),
   IPV6_POLICY("ipv6-policy"),
+  IPV6_READ_HISTORY("ipv6-read-history"),
+  IPV6_WRITE_HISTORY("ipv6-write-history"),
   KNOWN_FLAGS("known-flags"),
   LEGACY_DIR_KEY("legacy-dir-key"),
   LEGACY_KEY("legacy-key"),
@@ -101,6 +118,8 @@ public enum Key {
   ONION_KEY_CROSSCERT("onion-key-crosscert"),
   OPT("opt"),
   OR_ADDRESS("or-address"),
+  OVERLOAD_RATELIMITS("overload-ratelimits"),
+  OVERLOAD_FD_EXHAUSTED("overload-fd-exhausted"),
   P("p"),
   P6("p6"),
   PACKAGE("package"),
@@ -136,7 +155,13 @@ public enum Key {
   SIGNING_KEY("signing-key"),
   SNOWFLAKE_IDLE_COUNT("snowflake-idle-count"),
   SNOWFLAKE_IPS("snowflake-ips"),
+  SNOWFLAKE_IPS_BADGE("snowflake-ips-badge"),
+  SNOWFLAKE_IPS_NAT_RESTRICTED("snowflake-ips-nat-restricted"),
+  SNOWFLAKE_IPS_NAT_UNKNOWN("snowflake-ips-nat-unknown"),
+  SNOWFLAKE_IPS_NAT_UNRESTRICTED("snowflake-ips-nat-unrestricted"),
+  SNOWFLAKE_IPS_STANDALONE("snowflake-ips-standalone"),
   SNOWFLAKE_IPS_TOTAL("snowflake-ips-total"),
+  SNOWFLAKE_IPS_WEBEXT("snowflake-ips-webext"),
   SNOWFLAKE_STATS_END("snowflake-stats-end"),
   TRANSPORT("transport"),
   TUNNELLED_DIR_SERVER("tunnelled-dir-server"),
